package App::Greple;

use strict;
use warnings;

=head1 NAME

greple - extensible grep with lexical expression and region handling

=head1 VERSION

Version 8.3301

=cut

our $VERSION = '8.3301';

=head1 AUTHOR

Kazumasa Utashiro

=head1 LICENSE AND COPYRIGHT

Copyright (c) 1991-2017 Kazumasa Utashiro

Use and redistribution for ANY PURPOSE are granted as long as all copyright notices are retained. Redistribution with modification is allowed provided that you make your modified version obviously distinguishable from the original one. THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED.

=cut

1; # End of App::Greple
